#include "stdafx.h"
#include "FMOD.h"
#include <math.h>

FM::ChannelControl FM::ChannelGlobal("Global");
FM::ChannelControl FM::ChannelFX("FX");
FM::ChannelControl FM::ChannelMusic("Music");
FM::ChannelControl FM::ChannelAmbient("Ambient");
FM::ChannelControl FM::ChannelVoice("Voice");
FM::ChannelControl FM::ChannelGui("Gui");

Memc<FMOD::ChannelGroup*> FM::Mix;

void FM::ChannelControl::add(FM::ChannelControl& subChannel) {
	_channel->addGroup(subChannel._channel);
}

FM::ChannelControl::ChannelControl(Str name) {
	_name = name;
	_underwater = false;
	// add this channel to the system if it is initialized
	if (FM::SoundSystem.active()) {
		create();
	}
}

FMOD::ChannelGroup* FM::ChannelControl::operator()() {
	return _channel;
}

void FM::ChannelControl::create() {
	// add this channel to the system
	FM::SoundSystem.addChannelGroup(_name, _channel);
	// add all channels but global to global
	if (!Equal(_name,  "Global")) {
		FM::ChannelGlobal.add(T);
	} else {
		// add underwater effect to global channel
		FM::SoundSystem._createDSP(FMOD_DSP_TYPE_LOWPASS, &_lowpass);
		_lowpass->setBypass(true);
		_channel->addDSP(_lowpass, NULL);
	}

	// Add to container with all channels
	FM::Mix.add(_channel);
}

FM::ChannelControl& FM::ChannelControl::volume(Flt volume) {
	_errorCheck(_channel->setVolume(volume));
	return T;
}

Flt FM::ChannelControl::volume() {
	Flt volume;
	_channel->getVolume(&volume);
	return volume;
}

// lowpass for under water effect
void FM::ChannelControl::_lowpassFilter(Bool status, Flt depth) {
	if (status) { 
		if (!_underwater) {
			_underwater = true;
			_lowpass->setBypass(false);
		}
		// attenuate cutoff according to depth
		if (depth > 19) depth = 19;
		_lowpass->setParameter(FMOD_DSP_LOWPASS_CUTOFF, 400.0 - (log10(depth) * 200));
	} else if (_underwater) {
		_underwater = false;
		_lowpass->setBypass(true);
	}
}


// outside class
FMOD::ChannelGroup* FM::Channel(C Str &name) {
	for (int i = 0; i < FM::Mix.elms(); i++) {
		char channelName[100];
		FM::Mix[i]->getName(channelName, 100);
		if (Equal(channelName, name)) {
			return FM::Mix[i];
		}
	}
	return NULL;
}

