/******************************************************************************
 * Copyright (c) yvan vander sanden.												                  *
 * Header file to wrap FMOD sound library in Esenthel Engine                  *
/******************************************************************************/

// Standard channels are FX, Gui, Music, Ambient, Voice and Global.
// But you can make as many as you want. All will be added to the Global channel.

#pragma once

namespace FM {
	struct ChannelControl {
		ChannelControl& volume(Flt volume); Flt volume();

		// constructor
		ChannelControl(Str name);

		// operators
		FMOD::ChannelGroup * operator()();

		/**********************************/
		/* following are for internal use */
		/**********************************/

		// mechanism to add channels to each other
		void add(ChannelControl& subChannel);
		// this will be called by SoundSystem when channel is added
		void create();
		// for enabling lowpass filter on global channel (for underwater effect)
		void _lowpassFilter(Bool status, Flt depth = 0);

	private:
		FMOD::ChannelGroup * _channel;
		Str _name;
		// lowpass filter to improve underwater effect on global channel
		Bool _underwater;
		FMOD::DSP *_lowpass;
	};

	extern ChannelControl ChannelFX;
	extern ChannelControl ChannelMusic;
	extern ChannelControl ChannelAmbient;
	extern ChannelControl ChannelVoice;
	extern ChannelControl ChannelGui;
	extern ChannelControl ChannelGlobal;

	// We also need a global container with pointers to all channels.
	// This is the only way to find the channel by name.
	extern Memc<FMOD::ChannelGroup*> Mix;
	// And now a function to find a channel by name
	FMOD::ChannelGroup* Channel(C Str &name);

}