#include "stdafx.h"
#include "FMOD.h"

/*******************************************************/
/*            global objects                           */

FM::FaderControl FM::Fader;

/*******************************************************/
/*            Channel Fader Control                    */

void FM::FaderControl::add(FMOD::Channel * channel, Flt time, Flt target, Bool deleteChannel) {
	// make sure this channel is not already faded
	remove(channel);

	FaderItem fader;
	fader.channel = channel				;
	fader.target	= target				;
	fader.Delete	= deleteChannel	;

	Flt volume;
	channel->getVolume(&volume);
	Flt step = target - volume;
	fader.step = step;

	_faders.add(fader);
}

void FM::FaderControl::remove(FMOD::Channel * channel) {
	// remove this channel if it exists
	Int i = 0;
	while (i < _faders.elms()) {
		if (_faders[i].channel == channel) {
			if (_faders[i].Delete) {
				_faders[i].channel->stop();
			}
			_faders.remove(i);
		} else {
			i++;
		}
	}
}

void FM::FaderControl::update() {
	Int i = 0;
	while (i < _faders.elms()) {
		Flt volume;
		_faders[i].channel->getVolume(&volume);
		volume += (_faders[i].step * Time.rd());
		SAT(volume);
		_faders[i].channel->setVolume(volume);
		if (Abs(_faders[i].target - volume) < Abs(_faders[i].step * Time.rd() * 2)) {
			_faders[i].channel->setVolume(_faders[i].target);
			_faders.remove(i);
		} else {
			i++;
		}
	}
}