#include "stdafx.h"
#include "FMOD.h"


FM::geometryObject& FM::geometryObject::active(Bool status) {
	_object->setActive(status);
	return T;
}

FM::geometryObject& FM::geometryObject::pos(C Vec& position) {
	_object->setPosition(&FM::vecToFmod(position));
	return T;
}

FM::geometryObject& FM::geometryObject::forward(C Vec& forward) {
	FMOD_VECTOR f = vecToFmod(forward);
	f.z = -(f.z);
	_object->setRotation(&f, NULL);
	return T;
}

FM::geometryObject& FM::geometryObject::up(C Vec &up) {
	_object->setRotation(NULL, &FM::vecToFmod(up));
	return T;
}


FM::geometryObject& FM::geometryObject::rotation(C Vec& forward, C Vec& up) {
	FMOD_VECTOR f = vecToFmod(forward);
	f.z = -(f.z);
	_object->setRotation(&f, &FM::vecToFmod(up));
	return T;
}

FM::geometryObject& FM::geometryObject::scale(C Vec& scale) {
	_object->setScale(&FM::vecToFmod(scale));
	return T;
}

Bool FM::geometryObject::active() {
	Bool result;
	_object->getActive(&result);
	return result;
}

Vec FM::geometryObject::pos() {
	FMOD_VECTOR pos;
	_object->getPosition(&pos);
	return FM::fmodToVec(pos);
}

void FM::geometryObject::draw(Color color) {
	// set drawing matrix
	FMOD_VECTOR pos, up, forward, x;
	_object->getPosition(&pos);
	_object->getRotation(&forward, &up);
	forward.z = -(forward.z);
	x.x = up.y * forward.z - up.z * forward.y;
	x.y = up.z * forward.x - up.x * forward.z;
	x.z = up.x * forward.y - up.y * forward.x;
	Matrix m;
	m.z = fmodToVec(forward);
	m.y = fmodToVec(up);
	m.x = fmodToVec(x);
	m.pos = fmodToVec(pos);
	SetMatrix(m);

	// draw all polygons
	Int polyIndex = 0;
	_object->getNumPolygons(&polyIndex);
	for (Int x = 0; x < polyIndex; x++) {
		Int vertices = 0;
		_object->getPolygonNumVertices(x, &vertices);

		Memc<Vec> poly;
		for (Int y = 0; y < vertices; y++) {
			FMOD_VECTOR vtx;
			_object->getPolygonVertex(x, y, &vtx);
			Vec &temp = poly.New();
			temp = fmodToVec(vtx);
		}
		DrawPoly(poly, color);
	}

	SetMatrix();
	m.draw();
}

void FM::geometryObject::release() {
	_object->release();
}