#include "stdafx.h"
#include "FMOD.h"

/*******************************************************/
/*            global objects                           */

FM::MusicManager FM::Music;
FM::MusicManager FM::Ambient("Ambient");




/*******************************************************/
/*            MusicTheme                               */

void FM::MusicTheme::operator+=(Str name) {
	UInt pos	= FM::SoundSystem.addSound(name);
	_songs.add(pos);
	_lastPos.add(0);
}

Str FM::MusicTheme::song(Int i) {
	return FM::SoundSystem.getSoundName(_songs(i));
}

Flt FM::MusicTheme::lastPosition(Int i) {
	return _lastPos(i);
}

void FM::MusicTheme::lastPosition(Int i, Flt pos) {
	_lastPos(i) = pos;
}

FM::MusicTheme::MusicTheme() {
	_cur = -1;
}




/********************************************************/
/*            MusicManager                              */

FM::MusicManager::MusicManager(Str channelgroup) {
	_newTheme				= NULL				;
	_channel_group	= channelgroup;
	fade_in					= 0.5					;
	fade_out				= 3.0					;
	time_reset			= 10.0				;
	_songID[0]			= -1					;
	_songID[1]			= -1					;
	_current				= 1						;
	playing					= false				;
}

Str FM::MusicManager::name() {
	return _newTheme->song(_songID[_current]);
}

Flt FM::MusicManager::time() {
	return _sound[_current].time();
}

Flt FM::MusicManager::length() {
	return _sound[_current].length();
}

void FM::MusicManager::set(FM::MusicTheme * theme) {
	_newTheme = theme;
}

void FM::MusicManager::play(FM::MusicTheme *theme, Int song) {
	_newTheme = theme;
	if (_newTheme == NULL) {
		stop();
	} else {
		if ((song > _newTheme->songs()) || (song == -1)) {
			song = Random(_newTheme->songs());
		}
		_playNow(song);
	}
}

void FM::MusicManager::stop() {
	if (playing) {
		_sound[_current].fadeOut(fade_out);
		_themeOfPlaying->lastPosition(_songID[_current], _sound[_current].time());
		playing = false;
	}
}

void FM::MusicManager::next() {
	// choose a new song	
	Int song = _songID[_current];

	// if the theme has not changed and has more than one title,
	// be extra carefull not to play the same thing twice
	if (_newTheme == _themeOfPlaying) {
		if (_newTheme->songs() > 1) {
			while (song == _songID[_current]) {
				song = Random(_newTheme->songs());
			}
		}
	} else {
		// other theme, just pick something
		song = Random(_newTheme->songs());
	}
	_playNow(song);
}

void FM::MusicManager::_playNow(Int song) {
	if (playing) {
		// fade current song
		_sound[_current].fadeOut(fade_out, true);
		_themeOfPlaying->lastPosition(_songID[_current], _sound[_current].time());
	}

	// switch current
	_current = !_current;

	// delete any sound that is still playing here, just in case
	_sound[_current].del();
	_songID[_current] = song;

	// start playing
	_sound[_current].create(_newTheme->song(_songID[_current]), false, 0, Channel(_channel_group), false, true);

	// forward to last played time
	if ((_sound[_current].length() - _newTheme->lastPosition(_songID[_current])) 
		> time_reset) {
			_sound[_current].time(_newTheme->lastPosition(_songID[_current]));
	}

	_sound[_current].play();
	_sound[_current].fadeIn(fade_in);
	_themeOfPlaying = _newTheme;
	playing = true;
}


