/******************************************************************************
 * Copyright (c) yvan vander sanden.												                  *
 * Header file to wrap FMOD sound library in Esenthel Engine                  *
/******************************************************************************/

#pragma once

namespace FM {
	struct MusicTheme { // Set of Music Tracks !! must be stored in constant memory address !!
		// get
		Int songs(     ) {return _songs.elms();}	// get number   of      songs in this theme
		Str song (Int i);													// get the name of i-th song  in this theme
		Flt		lastPosition(Int i					);			// get the last played position of the i-th song when stopped by musicmanager
		void	lastPosition(Int i, Flt pos	);			// set

		// operations
		void operator+=(Str name); // add song to theme, 'name'=song file name
		MusicTheme();

	private:
		Int       _cur;
		Memc<Int> _songs;
		Memc<Flt> _lastPos; // remember position at which this song was last stopped
	};

	/******************************************************************************/
	struct MusicManager {
		Flt		fade_in   , // fade in  time for starting  new songs, default=0.5
					fade_out	, // fade out time for finishing old songs, default=3.0
					time_reset; // minimum  time left needed to play a song again from its last position instead of playing it from the start, default=10
		Bool	playing		; // check if playing

		// get
		Str         name  ();												// current song name
		Flt         time  ();												// current song time position
		Flt         length();												// current song length
		MusicTheme* theme () {return T._newTheme;}	// active  theme

		// set
		void set (MusicTheme *theme             );                     // set active   music theme, this theme will be used after current song finishes playing
		void set (MusicTheme &theme             ) {set (&theme     );} // set active   music theme, this theme will be used after current song finishes playing
		void play(MusicTheme *theme, Int song=-1);                     // set and play music theme with 'song' i-th song in the theme (-1=random)
		void play(MusicTheme &theme, Int song=-1) {play(&theme,song);} // set and play music theme with 'song' i-th song in the theme (-1=random)
		void stop(); // stop playing this musicmanager
		void next(); // skip to the next random song from active theme
		
		MusicManager(Str channelgroup = "Music");

	private:
		Int						_songID	[2]			;
		Sound					_sound	[2]			;
		Str						_channel_group	;
		MusicTheme*		_newTheme				;
		MusicTheme*		_themeOfPlaying	;

		Bool					_current				;

		void _playNow(Int song)				;
	}; 
	
	extern MusicManager Music  ;  // Music   MusicManager with VOLUME_MUSIC   sound VOLUME_GROUP
	extern MusicManager	Ambient;  // Ambient MusicManager with VOLUME_AMBIENT sound VOLUME_GROUP
	/******************************************************************************/
}