#include "stdafx.h"
#include "FMOD.h"

FM::SoundStream& FM::SoundStream::del() {
	Fader.remove(_channel);
	_channel->stop();
	_channel = NULL;
	return T;
}

FM::SoundStream&	FM::SoundStream::create(Str name,	Bool loop, Flt volume, FMOD::ChannelGroup * channelgroup) {
	_ID = FM::SoundSystem.addStream(name);

	_currentSound = FM::SoundSystem.getSoundPtr(_ID);
	if (_currentSound != NULL) {
		FM::SoundSystem.addChannel(_currentSound, _channel);
		if (channelgroup == NULL) channelgroup = ChannelFX();
		_channel->setChannelGroup(channelgroup);
		if (loop) _channel->setLoopCount(-1);
		else _channel->setLoopCount(0);
		_channel->setVolume(volume);
		_channel->setMode(FMOD_2D);
	}
	_is3D = false;
	_speed = 1.0;

	// get channel name for save file
	char channel_name[100];
	channelgroup->getName(channel_name, 100);
	_channel_group = channel_name;
	return T;
}

FM::SoundStream&	FM::SoundStream::create(Str name,	C Vec &pos, Flt range, Bool loop, Flt volume, FMOD::ChannelGroup * channelgroup) {
	_ID = FM::SoundSystem.addStream(name);

	_currentSound = FM::SoundSystem.getSoundPtr(_ID);
	if (_currentSound != NULL) {
		FM::SoundSystem.addChannel(_currentSound, _channel);
		if (channelgroup == NULL) channelgroup = ChannelFX();
		_channel->setChannelGroup(channelgroup);
		if (loop) _channel->setLoopCount(-1);
		else _channel->setLoopCount(0);
		_channel->setVolume(volume);
		_channel->setMode(FMOD_3D);
	
		// add 3D specific stuff
		_channel->set3DMinMaxDistance(range, 10000.0);
		FMOD_VECTOR position = FM::vecToFmod(pos);
		FMOD_VECTOR velocity;
		_lastPos = position;
		velocity.x = 0; velocity.y = 0; velocity.z = 0;
		_channel->set3DAttributes(&position, &velocity);

		// spread stereo sounds in 3D
		int channels;
		_currentSound->getFormat(NULL, NULL, &channels, NULL);
		if (channels > 1) {
			Flt spread = channels * 45;
			// limit this a bit
			if (spread > 270) spread = 270;
			_channel->set3DSpread(spread);
		}
	}
	_is3D = true;
	_speed = 1.0;

	// get channel name for save file
	char channel_name[100];
	channelgroup->getName(channel_name, 100);
	_channel_group = channel_name;

	return T;
}

FM::SoundStream& FM::SoundStream::stop() {
	_channel->setPaused(true);
	_channel->setPosition(0, FMOD_TIMEUNIT_MS);
	return T;
}

FM::SoundStream& FM::SoundStream::pause() {
	_channel->setPaused(true);
	return T;
}

Bool FM::SoundStream::play() {
	FMOD_RESULT result = _channel->setPaused(false);
	return true;
}

Bool FM::SoundStream::playing() {
	Bool isplaying;
	_channel->isPlaying(&isplaying);
	return isplaying;
}

Bool FM::SoundStream::ready() {
	FMOD_OPENSTATE result;
	FMOD_RESULT test = _currentSound->getOpenState(&result, NULL, NULL, NULL);
	if ((result == FMOD_OPENSTATE_READY)||(result == FMOD_OPENSTATE_PLAYING)) return true;
	return false;
}

void FM::SoundStream::netStatistics(Str &state, UInt &buffered, Bool &starving) {
	FMOD_OPENSTATE result;
	_currentSound->getOpenState(&result, &buffered, &starving, NULL);
	switch(result) {
	case FMOD_OPENSTATE_READY: state = "ready"; break;
	case FMOD_OPENSTATE_LOADING: state = "loading"; break;
	case FMOD_OPENSTATE_ERROR: state = "error"; break;
	case FMOD_OPENSTATE_CONNECTING: state = "connecting"; break;
	case FMOD_OPENSTATE_BUFFERING: state = "buffering"; break;
	case FMOD_OPENSTATE_SEEKING: state = "seeking"; break;
	case FMOD_OPENSTATE_PLAYING: state = "playing"; break;
	case FMOD_OPENSTATE_SETPOSITION: state = "playing"; break;
	}
}

Str FM::SoundStream::name() {
	return FM::SoundSystem.getSoundName(_ID);
}

FM::SoundStream& FM::SoundStream::volume(Flt volume) {
	_channel->setVolume(volume);
	return T;
}

Flt FM::SoundStream::volume() {
	Flt volume;
	_channel->getVolume(&volume);
	return volume;
}

FM::SoundStream& FM::SoundStream::spread(Flt spread) {
	if (spread > 360) spread = 360;
	if (spread < 0	) spread = 0	;

	_channel->set3DSpread(spread);
	return T;
}

Flt FM::SoundStream::spread() {
	Flt s;
	_channel->get3DSpread(&s);
	return s;
}

FM::SoundStream& FM::SoundStream::range(Flt range) {
	if (_is3D) _channel->set3DMinMaxDistance(range, 10000.0);
	return T;
}

Flt FM::SoundStream::range() {
	if (_is3D) {
		Flt range;
		_channel->get3DMinMaxDistance(&range, NULL);
		return range;
	}
	return 0;
}

FM::SoundStream& FM::SoundStream::pos(C Vec &pos) {
	Bool playing;
	_channel->isPlaying(&playing);
	if(playing && _is3D) {
		FMOD_VECTOR position = FM::vecToFmod(pos);
		FMOD_VECTOR velocity;
		velocity.x = (position.x - _lastPos.x) * (1 / Time.d());
		velocity.y = (position.y - _lastPos.y) * (1 / Time.d());
		velocity.z = (position.z - _lastPos.z) * (1 / Time.d());
		_channel->set3DAttributes(&position, &velocity);
		_lastPos = position;
	}
	return T;
}

Vec FM::SoundStream::pos() {
	Vec pos;
	pos.set(_lastPos.x, _lastPos.y, _lastPos.z);
	return pos;
}

FM::SoundStream& FM::SoundStream::fadeInFromSilence(Flt fade_duration) {
	_channel->setVolume(0);
	Fader.add(_channel, fade_duration, 1);
	return T;
}

FM::SoundStream& FM::SoundStream::fadeIn(Flt fade_duration) {
	Fader.add(_channel, fade_duration, 1);
	return T;
}

FM::SoundStream& FM::SoundStream::fadeOut(Flt fade_duration, Bool deleteChannel) {
	Fader.add(_channel, fade_duration, 0, deleteChannel);
	return T;
}

void FM::SoundStream::save(File &f) {
	if(_channel != NULL) {
		f.putStr(FM::SoundSystem.getSoundName(_ID));
		f << _is3D;
		f << pos();
		f << playing();
//		f << raw();
//		f << loop();
		f << volume();
//		f << speed();
		f << range();
		f << _channel_group;
	}
}

Bool FM::SoundStream::load(File &f) {
	_channel->stop();
	Str name;
	Vec pos;
	Flt raw, volume, speed, range;
	Bool loop, playing;
	f.getStr(name);
	f >> _is3D;
	f >> pos;
	f >> playing;
	f >> raw;
	f >> loop;
	f >> volume;
	f >> speed;
	f >> range;
	f >> _channel_group;

	// try to find channelpointer from name

	if (_is3D) {
		T.create(name, pos, range, loop, volume, FM::Channel(_channel_group));
	} else {
		T.create(name, loop, volume, FM::Channel(_channel_group));
	}
	//T.raw(raw);
	//T.speed(speed);
	if (playing) T.play();

	return true;
}