/******************************************************************************
 * Copyright (c) yvan vander sanden.												                  *
 * Header file to wrap FMOD sound library in Esenthel Engine                  *
/******************************************************************************/

#pragma once


namespace FM {
	struct SoundStream {
		// manage
		FM::SoundStream&	del();
		FM::SoundStream&	create(Str name,													Bool loop=0, Flt volume=1, FMOD::ChannelGroup * channelgroup = NULL);
		FM::SoundStream&	create(Str name, C Vec &pos, Flt range=1, Bool loop=0, Flt volume=1, FMOD::ChannelGroup * channelgroup = NULL);
		Bool							play  (Str name,                          Bool loop=0, Flt volume=1, FMOD::ChannelGroup * channelgroup = NULL) {return create(name,          loop,volume,channelgroup).play();} // create and play    sound, 'name'=sound file name,                                                                       , 'loop'=if sound is looped, 'volume'=sound volume (0..1)
		Bool							play  (Str name, C Vec &pos, Flt range=1, Bool loop=0, Flt volume=1, FMOD::ChannelGroup * channelgroup = NULL) {return create(name,pos,range,loop,volume,channelgroup).play();} // create and play 3D sound, 'name'=sound file name, 'pos'=sound position in World, 'range'=sound range multiplier (0..Inf), 'loop'=if sound is looped, 'volume'=sound volume (0..1)

		// stop / play
		SoundStream& stop();
		SoundStream& pause();
		Bool play ();
		Bool ready();
		void netStatistics(Str &state, UInt &buffered, Bool &starving);

		// get / set
																					Bool playing(); // if playing
																					Str  name   (); //     get stream name
	//																				Int  raws   (); //     get stream raw  length (number of bytes)
	//																				Flt  length (); //     get stream time length (in seconds)
//		SoundStream& raw   (  Int  raw   );   Int  raw    (); // set/get stream raw      position  , 0..raws  ()
//		SoundStream& time  (  Flt  t     );   Flt  time   (); // set/get stream time     position  , 0..length()
//		SoundStream& frac  (  Flt  f     );   Flt  frac   (); // set/get stream fraction position  , 0..1
//		SoundStream& loop  (  Bool loop  );   Bool loop   (); // set/get stream looping            , true/false
		SoundStream& volume(  Flt  volume);   Flt  volume (); // set/get stream volume             , 0..1
//		SoundStream& speed (  Flt  speed );   Flt  speed  (); // set/get stream speed              , 0.., default=1, negative speed plays the sound backwards
		SoundStream& spread(	Flt	 spread);		Flt	 spread	(); // set/get 3D spread in speaker space to 0..360 degrees
		// range with logarithmic rolloff (the default) is really the minimum distance before a stream
		// starts attenuating. This means it is at maximum volume until that point. 
		// See FMOD manual about logarithmic distance model for more about this.
		SoundStream& range (  Flt  range );   Flt  range  (); // set/get sound 3D range multiplier, 0..Inf
		SoundStream& pos   (C Vec &pos   );   Vec  pos    (); // set/get sound 3D position
		// velocity is calculated from last and new position
		//Sound& vel   (C Vec &vel   );   Vec  vel    (); // set/get sound 3D velocity

		// fade
		SoundStream& fadeInFromSilence(Flt fade_duration); // this method sets the fading value to silence, and then fades in the sound smoothly
		SoundStream& fadeIn           (Flt fade_duration); // fades in  the sound smoothly
		SoundStream& fadeOut          (Flt fade_duration, Bool deleteChannel = false); // fades out the sound smoothly

		// io
		void save(File &f); // save stream settings (name, raw, pos, volume, ...)
		Bool load(File &f); // load stream settings (name, raw, pos, volume, ...), doesn't play the sound automatically

	 ~SoundStream() {del();}
		SoundStream() {_channel=NULL;}

	private:
		FMOD::Channel *		_channel			;
		FMOD::Sound		*		_currentSound	;
		FMOD_VECTOR				_lastPos			;
		UInt							_ID						;
		Bool							_is3D					;
		Flt								_speed				;
		Str								_channel_group;
	};

}