#include "stdafx.h"
#include "System.h"
#include "dsp.h"
#include "dsp/dspElement.h"


void FM::dsp::create() {
	memset(&description, 0, sizeof(FMOD_DSP_DESCRIPTION));

	Set(description.name, "DSP patcher", 32);
	description.channels = 0;
	description.read = readCallback;
	description.userdata = this;

	FM::SoundSystem._createDSP(description, &dspPtr);
	dspPtr->setBypass(false);
}

FMOD_RESULT F_CALLBACK FM::dsp::readCallback(FMOD_DSP_STATE * dsp_state, float *in, float *out, unsigned int length, int inchannels, int outchannels) {
	void * userdata;
	((FMOD::DSP *)dsp_state->instance)->getUserData(&userdata);
	FM::dsp * object = (FM::dsp *)userdata;
	
	object->in(in, length);
	object->out(out, length);

	FREPA(object->dspElements) {
		switch(object->dspElements[i].data->type) {
			case DSP::DSP_RINGMODULATOR: {
				DSP::ringModulator * r = (DSP::ringModulator*)object->dspElements[i].data;
				r->process(object->in);

			}
			// add your own elements here

		}
	}

	object->out.copy(object->in);
	return FMOD_OK;
}

FM::dsp& FM::dsp::byPass(Bool bp) {
	dspPtr->setBypass(bp);
	return T;
}

Bool FM::dsp::byPass() {
	Bool * r;
	dspPtr->getBypass(r);
	return *r;
}

FM::DSP::dspElmPtr * FM::dsp::add(FM::DSP::DSP_TYPE type, Flt order) {
	switch (type) {
		case DSP::DSP_RINGMODULATOR: {
			DSP::ringModulator & tmp = DSP::RingModulators.New();
			DSP::dspElmPtr & dspElm = dspElements.New();
			dspElm.data = (DSP::dspElement*) &tmp;
			dspElm.order = order;
			return & dspElm;
		}
		// Add your own modules here
	}
}

void FM::dsp::remove(DSP::dspElmPtr * element) {
	FREPA(dspElements) {
		if (&dspElements[i] == element) {
			switch(dspElements[i].data->type) {
				case DSP::DSP_RINGMODULATOR: DSP::RingModulators.removeData((DSP::ringModulator*)(element->data)); break;
			}
			// your own modules here

			dspElements.remove(i);
			break;
		}
	}
}

static Int CompareDspElms(C FM::DSP::dspElmPtr &a, C FM::DSP::dspElmPtr &b) {
	return Sign(a.order - b.order);
}

void FM::dsp::sort() {
	dspElements.sort(CompareDspElms);
}


