#include "stdafx.h"
#include "delay.h"


#define XTRASAMPS 4
#define DEFDELVS 64
#define SAMPBLK 4

FM::DSP::delay::delay() {
	bufferlength = 0;
	buffer = Alloc<Flt>(XTRASAMPS);
}

FM::DSP::delay& FM::DSP::delay::set(UInt size) {
	T.size = size;
	return T;
}

FM::DSP::delay& FM::DSP::delay::update(samplePtr &s) {
	{
		Int nsamps = size * sampleRate * 0.001f;
		if (nsamps < 1) nsamps = 1;
		nsamps += ((- nsamps) & (SAMPBLK - 1));
		nsamps += DEFDELVS;
		if (T.bufferlength != nsamps) {
			Realloc(buffer, (nsamps + XTRASAMPS), bufferlength + XTRASAMPS);
			bufferlength = nsamps;
			phase = XTRASAMPS;
		}
	}

	currentLength = s.getLength();
	Flt * in = s.getBuffer();
	UInt length = currentLength;
	Int ph = phase;
	Flt * vp = buffer;
	Flt * bp = vp + ph;
	Flt * ep = vp + bufferlength + XTRASAMPS;
	ph += currentLength;
	
	while (length--) {
		Flt f = *in++;
		*bp++ = f;
		if (bp == ep) {
			vp[0] = ep[-4];
			vp[1] = ep[-3];
			vp[2] = ep[-2];
			vp[3] = ep[-1];
			bp = vp + XTRASAMPS;
			ph -= bufferlength;
		}
	}
	bp = vp + phase;
	phase = ph;

	return T;
}

FM::DSP::delay& FM::DSP::delay::get(sample& s, UInt delayTime) {
	if (s.getLength() < currentLength) s.resize(currentLength);
	UInt delaySamples = (UInt)(sampleRate * delayTime * 0.001f) + currentLength;
	if (delaySamples < s.getLength()) delaySamples = s.getLength();
	else if (delaySamples > bufferlength - DEFDELVS) delaySamples = bufferlength - DEFDELVS;

	Int ph = phase - delaySamples;

	Flt * vp = buffer;
	Flt * ep = vp + (bufferlength + XTRASAMPS);
	if (ph < 0) ph += bufferlength;
	Flt *bp = vp + ph;

	Flt * out = s.getBuffer();

	Int length = currentLength;
	while (length--) {
		*out++ = *bp++;
		if (bp == ep) bp -= bufferlength;
	}

	return T;
}

