#include "stdafx.h"
#include "envelope.h"
#include "math_functions.h"

FM::DSP::envelope::envelope() {
	bufferExists = false;
	set();
}

FM::DSP::envelope::~envelope() {
	if (bufferExists) Free(buffer);
}

FM::DSP::envelope& FM::DSP::envelope::set(Int window, Int period) {
	if (T.window == window && T.period == period) return T;

	T.window = window;
	T.period = period;
	if (T.window < 1) T.window = 1024;
	if (T.period < 1) T.period = T.window / 2;
	if (T.period < T.window / MAXOVERLAP + 1) T.period = T.window / MAXOVERLAP + 1;

	if (bufferExists) Free(buffer);
	buffer = Alloc<Flt>(T.window + INITVSTAKEN);
	bufferExists = true;

	phase = 0;
	Int i;
	Zero(sumBuffer, MAXOVERLAP);
	Zero(buffer, T.window + INITVSTAKEN);
	for (i = 0; i < T.window; i++) buffer[i] = (1. - Cos((2 * 3.14159 * i) / T.window)) / T.window;

	currentBufferSize = INITVSTAKEN;
	power = 0;
	return T;
}

Flt FM::DSP::envelope::measure(samplePtr& s) {
	Flt * input = s.getBuffer();
	if (period % s.getLength()) realPeriod = period + s.getLength() - (period % s.getLength());
	else realPeriod = period;
	if (s.getLength() > currentBufferSize) {
		ReallocZero(buffer, window + s.getLength(), window + currentBufferSize);
		currentBufferSize = s.getLength();
	}

	Flt * sumPtr = sumBuffer;
	input += s.getLength();
	for (Int count = phase; count < window; count += realPeriod, sumPtr++) {
		Flt * hp = buffer + count;
		Flt * fp = input;
		Flt sum = *sumPtr;

		for (Int i = 0; i < s.getLength(); i++) {
			fp--;
			sum += *hp++ * (*fp * *fp);
		}
		*sumPtr = sum;
	}

	sumPtr[0] = 0;
	phase -= s.getLength();
	if (phase < 0) {
		power = sumBuffer[0];
		sumPtr = sumBuffer;
		for (Int count = realPeriod; count < window; count += realPeriod, sumPtr++) sumPtr[0] = sumPtr[1];
		sumPtr[0] = 0;
		phase = realPeriod - s.getLength();
	}
	return powToDb(power);
}

Flt FM::DSP::envelope::operator()() {
	return powToDb(power);
}