#pragma once
#include "sample.h"

namespace FM {
	namespace DSP {

		struct highPass {
			highPass& setFrequency(Flt f);
			samplePtr& operator()(samplePtr& in);
			highPass();

		private:
			Flt freq;
			Flt coef;
			Flt last;

			sample buffer;
		};


		struct lowPass {
			lowPass& setFrequency(Flt f);
			samplePtr& operator()(samplePtr& in);
			lowPass();

		private:
			Flt freq;
			Flt coef;
			Flt last;

			sample buffer;
		};

		struct bandPass {
			bandPass& set(Flt freq, Flt q);
			bandPass& setFrequency(Flt freq);
			bandPass& setQ(Flt q);
			samplePtr& operator()(samplePtr& in);
			bandPass();

		private:
			void calcCoef();
			static float qCos(Flt omega);
			Flt freq;
			Flt q;
			Flt gain;
			Flt last;
			Flt previous;
			Flt coef1, coef2;
			sample buffer;
		};

		enum BQ_TYPE {
			BQ_LOWPASS,
			BQ_HIGHPASS,
			BQ_BANDPASS,
			BQ_NOTCH,
			BQ_PEAK,
			BQ_LOWSHELF,
			BQ_HIGHSHELF,
		};

		// for visualizing biquad parameters, see: http://www.earlevel.com/main/2010/12/20/biquad-calculator/
		struct biQuad {
			biQuad& set(BQ_TYPE type, Flt frequency, Flt Q, Flt peakGain = 4);
			biQuad& setType	(BQ_TYPE type	) { T.type			= type			; calc(); return T;}
			biQuad& setFreq	(Flt frequency) { T.freq			= frequency	; calc(); return T;}
			biQuad& setQ		(Flt Q				) { T.q					= Q					; calc(); return T;}
			biQuad& setPeak	(Flt peakGain	) { T.peakGain	= peakGain	; calc(); return T;}

			// use this for masochism
			biQuad& setRaw	(Flt fb1, Flt fb2, Flt ff1, Flt ff2, Flt ff3);
			samplePtr& operator()(samplePtr& in);
			biQuad();

		private:
			void calc();
			Flt fb1, fb2, ff1, ff2, ff3;
			BQ_TYPE type;
			Flt freq;
			Flt q;
			Flt peakGain;

			Flt last;
			Flt previous;

			sample buffer;
		};

		struct sampleHold {
			sampleHold& reset	(Flt value = 1e20	) { lastIn	= value; return T; }
			sampleHold& set		(Flt value				) { lastOut = value; return T; }
			samplePtr& operator()(samplePtr& in, samplePtr& signal);
			sampleHold();

		private:
			Flt lastIn;
			Flt lastOut;
			sample buffer;
		};

		// vcf is in oscillators because it share a lot of that code



	}
}