#include "stdafx.h"
#include "ramp.h"


FM::DSP::ramp::ramp() {
	ticksLeft = 0;
	reTarget = false;
	current = target = time = 0;
}

FM::DSP::ramp& FM::DSP::ramp::set(Flt target, Int time) {
	if (time <= 0) {
		T.target = T.current = target;
		T.ticksLeft = 0;
		T.reTarget = false;
	} else {
		T.target = target;
		T.reTarget = true;
		T.time = time;
	}
	return T;
}

FM::DSP::ramp& FM::DSP::ramp::stop() {
	T.target = T.current;
	T.ticksLeft = 0;
	T.reTarget = false;
	return T;
}

FM::DSP::ramp& FM::DSP::ramp::update(samplePtr& s) {
	if (T.length < s.getLength()) T.resize(s.getLength());

	if (reTarget) {
		Int nTicks = time * _dspTickToMSEC;
		if (!nTicks) nTicks = 1;
		ticksLeft = nTicks;
		bigInc = (target - current) / (Flt)nTicks;
		inc = _1overN * bigInc;
		reTarget = false;
	}

	Int l = length;
	Flt * ptr = buffer;
	if (ticksLeft) {
		Flt f = current;
		while (l--) *ptr++ = f, f += inc;
		current += bigInc;
		ticksLeft--;
	} else {
		current = target;
		for (; l > 7; l -= 8, ptr += 8) {
			ptr[0] = current; ptr[1] = current; ptr[2] = current; ptr[3] = current;
			ptr[4] = current; ptr[5] = current; ptr[6] = current; ptr[7] = current;
		}
		while (l--) *ptr++ = current;
	}


	_1overN = 1. / length;
	_dspTickToMSEC = sampleRate / (1000. * length);

	return T;
}

Flt * FM::DSP::ramp::operator()() {
	return buffer;
}

Flt * FM::DSP::ramp::getBuffer() {
	return buffer;
}

Flt FM::DSP::ramp::getValue() {
	return current;
}