/******************************************************************************/
#include "stdafx.h"
#include "../FMOD/FMOD.h"

/******************************************************************************/
FM::Sound sound;      // sound
Vec soundPos = Vec(0, 0, 10);

/******************************************************************************/
struct Obj {
	Mesh				mesh				;
	MaterialPtr boxMaterial	;
	PhysBody		phys				;
	Actor				actor				;

	FM::geometryObject * soundMesh;

	void create() {
		// create mesh from Box
		boxMaterial.require("mtrl/brick/0.mtrl");
		mesh.parts.New().base.create(Box(6,4,1), VTX_TEX0|VTX_NRM|VTX_TAN);
	  mesh.setMaterial(boxMaterial).setRender().setBox();
		phys.parts.New().createMesh(mesh);

		// create actor
		actor.create(phys);

		// and soundmesh
		soundMesh = &FM::SoundSystem.createGeometry(phys);
	}

	void draw() { mesh.draw(Matrix(actor.pos())); }

	void pos(Vec& pos) {
		actor.pos(pos);
		soundMesh->pos(pos);
	}

	Vec pos() { return actor.pos(); }

} wall;

void InitPre()
{
   App.name("Sound 3D");
   App.flag=APP_MS_EXCLUSIVE;
	 DataPath("../data");
   Paks.add("engine.pak");
	 FM::SoundSystem.create();
}
/******************************************************************************/
Bool Init()
{
	 Physics.create("../Installation/PhysX");
   sound.play("sound/water.ogg", soundPos, 1, true); // play looped 3D sound, volume=1, position='pos'
	 
	 wall.create();
	 wall.pos(Vec(0, 0, 10));
   return true;
}
/******************************************************************************/
void Shut()
{
}
/******************************************************************************/
Bool Update() {
	if(Kb.bp(KB_ESC))return false;
	CamHandle(0.1f,10,CAMH_ZOOM|CAMH_ROT);
	 
	// update listener parameters
	{
		FM::Listener.orn(Cam.matrix.z,Cam.matrix.y)  // set listener orientation (from camera)
								.pos(Cam.matrix.pos           ); // set listener position    (from camera)
	}

	// set sound and wall position
	Vec wallPos = wall.pos();
	if (Kb.b(KB_DOWN))	{ soundPos.z	-= 4 * Time.d();	} 
	if (Kb.b(KB_UP	))	{ soundPos.z	+= 4 * Time.d();	}
	if (Kb.b(KB_LEFT))	{	wallPos.x		-= 4 * Time.d();	} 
	if (Kb.b(KB_RIGHT)) { wallPos.x		+= 4 * Time.d();  }


	sound	.pos(soundPos	);
	wall	.pos(wallPos	);

	FM::SoundSystem.update();
	return true;
	}

	void Render() {
	switch(Renderer()) {
		case RM_PREPARE:	wall.draw();
											LightPoint(100, Vec(0,0,0)).add();
											break;
	}
}


/******************************************************************************/
void Draw() {
	Renderer(Render);

	SetMatrix();
	Ball(1,soundPos).draw(WHITE); // draw ball at sound position

	// draw sound geometry mesh for debugging
	wall.soundMesh->draw();

	// check occlusion result between listener and sound position
	Str text = S + "Direct occlusion: " + FM::SoundSystem.directOcclusion(Cam.matrix.pos, soundPos);
	text += S + " Reverb occlusion: " + FM::SoundSystem.reverbOcclusion(Cam.matrix.pos, soundPos);
	D.text(0, -0.8, text);

	D.text(0, -0.9, "Press up/down to move sound, left/right to move wall");
}
/******************************************************************************/
